package com.common.commonlib.net.manager

import com.common.commonlib.log.Logger.d
import com.common.commonlib.net.interceptor.HostInterceptor
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.adapter.rxjava3.RxJava3CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

/**
 * 网络请求管理器
 *
 * @author wangym
 * @since 2021/7/28
 */
object RequestManager {
    const val TAG = "RequestManager"
    private var logLevel = HttpLoggingInterceptor.Level.NONE

    /**
     * 默认超时时间(s)
     */
    private const val defaultTimeOut: Long = 5

    /**
     * 默认读取超时时间(s)
     */
    private const val defaultReadTimeOut: Long = 10

    private var mRetrofit: Retrofit? = null

    private val loggingInterceptor = HttpLoggingInterceptor {
        d(TAG, it)
    }

    /**
     * 创建retrofit
     *
     * @param service 类型
     * @param interceptors 拦截器
     * @param loaderType 请求类别
     */
    fun <T> create(service: Class<T>, interceptors: List<Interceptor>?, loaderType: String): T {
        init(interceptors, loaderType)
        return mRetrofit!!.create(service)
    }

    private fun init(interceptors: List<Interceptor>?, loaderType: String) {
        val builder: OkHttpClient.Builder = OkHttpClient.Builder()
        builder.connectTimeout(defaultTimeOut, TimeUnit.SECONDS)
        builder.readTimeout(defaultReadTimeOut, TimeUnit.SECONDS)

        // 动态HOST替换拦截器
        builder.addInterceptor(HostInterceptor())
        // 添加通用拦截器
        val interceptorList = CommonInterceptorManager.getInterceptorsByType(loaderType)
        if (!interceptorList.isNullOrEmpty()) {
            d(TAG, "got common interceptor list")
            for (item in interceptorList.values) {
                builder.addInterceptor(item)
            }
        }
        // 添加接口自定义的拦截器
        if (!interceptors.isNullOrEmpty()) {
            for (intercept in interceptors) {
                builder.addInterceptor(intercept)
            }
        }

        // 添加日志拦截器 日志拦截器，需要添加在所有拦截器之后
        loggingInterceptor.level = logLevel
        builder.addInterceptor(loggingInterceptor)

        mRetrofit = Retrofit.Builder()
            .client(builder.build())
            .addCallAdapterFactory(RxJava3CallAdapterFactory.create())
            .addConverterFactory(GsonConverterFactory.create())
            .baseUrl("https://www")
            .build()
    }

    /**
     * 打开完整日志
     */
    fun openFullLog() {
        logLevel = HttpLoggingInterceptor.Level.BODY
    }

    /**
     * 打开基础日志
     */
    fun openBasicLog() {
        logLevel = HttpLoggingInterceptor.Level.BASIC
    }

    /**
     * 打开头部日志
     */
    fun openHeadLog() {
        logLevel = HttpLoggingInterceptor.Level.HEADERS
    }
}